@@ -16,7 +16,7 @@ require 'gpgme'
 #     (0 ... $_.length).each do |i| $_[i] = ?0 end if $_
 #     system('stty echo')
 #   end
-#   puts
+#   $stderr.puts
 # end
 
 unless ENV['GPG_AGENT_INFO']
@@ -28,7 +28,7 @@ plain = 'test test test'
 puts("Plaintext:\n#{plain}")
 
 # Perform symmetric encryption on PLAIN.
-cipher = GPGME::encrypt(nil, plain, {:armor => true
+cipher = GPGME::encrypt(nil, plain, {:armor => true,
 			  # :passphrase_callback => method(:passfunc)
 			})
 puts("Ciphertext:\n#{cipher}")
