@@ -266,8 +266,8 @@ read_cb (void *handle, void *buffer, size_t size)
 {
   VALUE vcb = (VALUE)handle, vcbs, vhook_value, vbuffer;
 
-  vcbs = RARRAY(vcb)->ptr[0];
-  vhook_value = RARRAY(vcb)->ptr[1];
+  vcbs = RARRAY_PTR(vcb)[0];
+  vhook_value = RARRAY_PTR(vcb)[1];
 
   vbuffer = rb_funcall (vcbs, rb_intern ("read"), 2, vhook_value,
 			LONG2NUM(size));
@@ -282,8 +282,8 @@ write_cb (void *handle, const void *buffer, size_t size)
 {
   VALUE vcb = (VALUE)handle, vcbs, vhook_value, vbuffer, vnwrite;
 
-  vcbs = RARRAY(vcb)->ptr[0];
-  vhook_value = RARRAY(vcb)->ptr[1];
+  vcbs = RARRAY_PTR(vcb)[0];
+  vhook_value = RARRAY_PTR(vcb)[1];
   vbuffer = rb_str_new (buffer, size);
 
   vnwrite = rb_funcall (vcbs, rb_intern ("write"), 3,
@@ -297,8 +297,8 @@ seek_cb (void *handle, off_t offset, int whence)
   VALUE vcb = (VALUE)handle, vcbs, vhook_value, vpos;
   ID id_seek = rb_intern ("seek");
 
-  vcbs = RARRAY(vcb)->ptr[0];
-  vhook_value = RARRAY(vcb)->ptr[1];
+  vcbs = RARRAY_PTR(vcb)[0];
+  vhook_value = RARRAY_PTR(vcb)[1];
 
   if (rb_respond_to (vcbs, id_seek))
     {
@@ -566,8 +566,8 @@ passphrase_cb (void *hook, const char *uid_hint, const char *passphrase_info,
 {
   VALUE vcb = (VALUE)hook, vpassfunc, vhook_value;
 
-  vpassfunc = RARRAY(vcb)->ptr[0];
-  vhook_value = RARRAY(vcb)->ptr[1];
+  vpassfunc = RARRAY_PTR(vcb)[0];
+  vhook_value = RARRAY_PTR(vcb)[1];
 
   rb_funcall (vpassfunc, rb_intern ("call"), 5,
 	      vhook_value,
@@ -604,8 +604,8 @@ rb_s_gpgme_get_passphrase_cb (VALUE dummy, VALUE vctx, VALUE rpassfunc,
   VALUE vcb = rb_iv_get (vctx, "@passphrase_cb");
 
   /* No need to call gpgme_get_passphrase_cb. */
-  rb_ary_store (rpassfunc, 0, RARRAY(vcb)->ptr[0]);
-  rb_ary_store (rhook_value, 0, RARRAY(vcb)->ptr[1]);
+  rb_ary_store (rpassfunc, 0, RARRAY_PTR(vcb)[0]);
+  rb_ary_store (rhook_value, 0, RARRAY_PTR(vcb)[1]);
   return Qnil;
 }
 
@@ -614,8 +614,8 @@ progress_cb (void *hook, const char *what, int type, int current, int total)
 {
   VALUE vcb = (VALUE)hook, vprogfunc, vhook_value;
 
-  vprogfunc = RARRAY(vcb)->ptr[0];
-  vhook_value = RARRAY(vcb)->ptr[1];
+  vprogfunc = RARRAY_PTR(vcb)[0];
+  vhook_value = RARRAY_PTR(vcb)[1];
 
   rb_funcall (vprogfunc, rb_intern ("call"), 5, vhook_value,
 	      rb_str_new2 (what), INT2NUM(type), INT2NUM(current),
@@ -647,8 +647,8 @@ rb_s_gpgme_get_progress_cb (VALUE dummy, VALUE vctx, VALUE rprogfunc,
 			    VALUE rhook_value)
 {
   VALUE vcb = rb_iv_get (vctx, "@progress_cb");
-  rb_ary_store (rprogfunc, 0, RARRAY(vcb)->ptr[0]);
-  rb_ary_store (rhook_value, 0, RARRAY(vcb)->ptr[1]);
+  rb_ary_store (rprogfunc, 0, RARRAY_PTR(vcb)[0]);
+  rb_ary_store (rhook_value, 0, RARRAY_PTR(vcb)[1]);
   return Qnil;
 }
 
@@ -908,7 +908,7 @@ rb_s_gpgme_op_genkey_start (VALUE dummy, VALUE vctx, VALUE vparms,
 }
 
 static VALUE
-rb_s_gpgme_op_export (VALUE dummy, VALUE vctx, VALUE vpattern, VALUE vreserved,
+rb_s_gpgme_op_export (VALUE dummy, VALUE vctx, VALUE vpattern, VALUE vmode,
 		      VALUE vkeydata)
 {
   gpgme_ctx_t ctx;
@@ -922,14 +922,14 @@ rb_s_gpgme_op_export (VALUE dummy, VALUE vctx, VALUE vpattern, VALUE vreserved,
     rb_raise (rb_eArgError, "released ctx");
   UNWRAP_GPGME_DATA(vkeydata, keydata);
 
-  err = gpgme_op_export (ctx, StringValueCStr(vpattern), NUM2UINT(vreserved),
+  err = gpgme_op_export (ctx, StringValueCStr(vpattern), NUM2UINT(vmode),
 			 keydata);
   return LONG2NUM(err);
 }
 
 static VALUE
 rb_s_gpgme_op_export_start (VALUE dummy, VALUE vctx, VALUE vpattern,
-			    VALUE vreserved, VALUE vkeydata)
+			    VALUE vmode, VALUE vkeydata)
 {
   gpgme_ctx_t ctx;
   gpgme_data_t keydata;
@@ -943,10 +943,112 @@ rb_s_gpgme_op_export_start (VALUE dummy, VALUE vctx, VALUE vpattern,
   UNWRAP_GPGME_DATA(vkeydata, keydata);
 
   err = gpgme_op_export_start (ctx, StringValueCStr(vpattern),
-			       NUM2UINT(vreserved), keydata);
+			       NUM2UINT(vmode), keydata);
   return LONG2NUM(err);
 }
 
+static VALUE
+rb_s_gpgme_op_export_ext (VALUE dummy, VALUE vctx, VALUE vpattern, VALUE vmode,
+			  VALUE vkeydata)
+{
+  gpgme_ctx_t ctx;
+  gpgme_data_t keydata;
+  gpgme_error_t err;
+  const char **pattern;
+  int i;
+
+  CHECK_KEYLIST_NOT_IN_PROGRESS(vctx);
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+  pattern = ALLOC_N(const char *, RARRAY_LEN(vpattern));
+  for (i = 0; i < RARRAY_LEN(vpattern); i++)
+    pattern[i] = StringValueCStr(RARRAY_PTR(vpattern)[i]);
+  UNWRAP_GPGME_DATA(vkeydata, keydata);
+
+  err = gpgme_op_export_ext (ctx, pattern, NUM2UINT(vmode), keydata);
+  return LONG2NUM(err);
+}
+
+static VALUE
+rb_s_gpgme_op_export_ext_start (VALUE dummy, VALUE vctx, VALUE vpattern,
+			    VALUE vmode, VALUE vkeydata)
+{
+  gpgme_ctx_t ctx;
+  gpgme_data_t keydata;
+  gpgme_error_t err;
+  const char **pattern;
+  int i;
+
+  CHECK_KEYLIST_NOT_IN_PROGRESS(vctx);
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+  pattern = ALLOC_N(const char *, RARRAY_LEN(vpattern));
+  for (i = 0; i < RARRAY_LEN(vpattern); i++)
+    pattern[i] = StringValueCStr(RARRAY_PTR(vpattern)[i]);
+  UNWRAP_GPGME_DATA(vkeydata, keydata);
+
+  err = gpgme_op_export_ext_start (ctx, pattern, NUM2UINT(vmode), keydata);
+  return LONG2NUM(err);
+}
+
+#ifdef HAVE_GPGME_OP_EXPORT_KEYS
+static VALUE
+rb_s_gpgme_op_export_keys (VALUE dummy, VALUE vctx, VALUE vkeys,
+			   VALUE vmode, VALUE vkeydata)
+{
+  gpgme_ctx_t ctx;
+  gpgme_key_t *keys;
+  gpgme_data_t keydata;
+  gpgme_error_t err;
+  int i;
+
+  CHECK_KEYLIST_NOT_IN_PROGRESS(vctx);
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  keys = ALLOC_N(gpgme_key_t, RARRAY_LEN(vkeys) + 1);
+  for (i = 0; i < RARRAY_LEN(vkeys); i++)
+    UNWRAP_GPGME_KEY(RARRAY_PTR(vkeys)[i], keys[i]);
+
+  UNWRAP_GPGME_DATA(vkeydata, keydata);
+
+  err = gpgme_op_export_keys (ctx, keys, NUM2UINT(vmode), keydata);
+  return LONG2NUM(err);
+}
+
+static VALUE
+rb_s_gpgme_op_export_keys_start (VALUE dummy, VALUE vctx, VALUE vkeys,
+				 VALUE vmode, VALUE vkeydata)
+{
+  gpgme_ctx_t ctx;
+  gpgme_key_t *keys;
+  gpgme_data_t keydata;
+  gpgme_error_t err;
+  int i;
+
+  CHECK_KEYLIST_NOT_IN_PROGRESS(vctx);
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  keys = ALLOC_N(gpgme_key_t, RARRAY_LEN(vkeys) + 1);
+  for (i = 0; i < RARRAY_LEN(vkeys); i++)
+    UNWRAP_GPGME_KEY(RARRAY_PTR(vkeys)[i], keys[i]);
+
+  UNWRAP_GPGME_DATA(vkeydata, keydata);
+
+  err = gpgme_op_export_keys_start (ctx, keys, NUM2UINT(vmode), keydata);
+  return LONG2NUM(err);
+}
+#endif	/*HAVE_GPGME_OP_EXPORT_KEYS*/
+
 static VALUE
 rb_s_gpgme_op_import (VALUE dummy, VALUE vctx, VALUE vkeydata)
 {
@@ -983,6 +1085,54 @@ rb_s_gpgme_op_import_start (VALUE dummy, VALUE vctx, VALUE vkeydata)
   return LONG2NUM(err);
 }
 
+#ifdef HAVE_GPGME_OP_EXPORT_KEYS
+static VALUE
+rb_s_gpgme_op_import_keys (VALUE dummy, VALUE vctx, VALUE vkeys)
+{
+  gpgme_ctx_t ctx;
+  gpgme_key_t *keys;
+  gpgme_error_t err;
+  int i;
+
+  CHECK_KEYLIST_NOT_IN_PROGRESS(vctx);
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  keys = ALLOC_N(gpgme_key_t, RARRAY_LEN(vkeys) + 1);
+  for (i = 0; i < RARRAY_LEN(vkeys); i++)
+    UNWRAP_GPGME_KEY(RARRAY_PTR(vkeys)[i], keys[i]);
+  keys[i] = NULL;
+
+  err = gpgme_op_import_keys (ctx, keys);
+  return LONG2NUM(err);
+}
+
+static VALUE
+rb_s_gpgme_op_import_keys_start (VALUE dummy, VALUE vctx, VALUE vkeys)
+{
+  gpgme_ctx_t ctx;
+  gpgme_key_t *keys;
+  gpgme_error_t err;
+  int i;
+
+  CHECK_KEYLIST_NOT_IN_PROGRESS(vctx);
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  keys = ALLOC_N(gpgme_key_t, RARRAY_LEN(vkeys) + 1);
+  for (i = 0; i < RARRAY_LEN(vkeys); i++)
+    UNWRAP_GPGME_KEY(RARRAY_PTR(vkeys)[i], keys[i]);
+  keys[i] = NULL;
+
+  err = gpgme_op_import_keys_start (ctx, keys);
+  return LONG2NUM(err);
+}
+#endif	/*HAVE_GPGME_OP_EXPORT_KEYS*/
+
 static VALUE
 rb_s_gpgme_op_import_result (VALUE dummy, VALUE vctx)
 {
@@ -1067,8 +1217,9 @@ edit_cb (void *hook, gpgme_status_code_t status, const char *args, int fd)
 {
   VALUE vcb = (VALUE)hook, veditfunc, vhook_value;
 
-  veditfunc = RARRAY(vcb)->ptr[0];
-  vhook_value = RARRAY(vcb)->ptr[1];
+
+  veditfunc = RARRAY_PTR(vcb)[0];
+  vhook_value = RARRAY_PTR(vcb)[1];
 
   rb_funcall (veditfunc, rb_intern ("call"), 4, vhook_value, INT2FIX(status),
 	      rb_str_new2 (args), INT2NUM(fd));
@@ -1919,11 +2070,27 @@ Init_gpgme_n (void)
   rb_define_module_function (mGPGME, "gpgme_op_export",
 			     rb_s_gpgme_op_export, 4);
   rb_define_module_function (mGPGME, "gpgme_op_export_start",
-			     rb_s_gpgme_op_export_start, 3);
+			     rb_s_gpgme_op_export_start, 4);
+  rb_define_module_function (mGPGME, "gpgme_op_export_ext",
+			     rb_s_gpgme_op_export_ext, 4);
+  rb_define_module_function (mGPGME, "gpgme_op_export_ext_start",
+			     rb_s_gpgme_op_export_ext_start, 4);
+#ifdef HAVE_GPGME_OP_EXPORT_KEYS
+  rb_define_module_function (mGPGME, "gpgme_op_export_keys",
+			     rb_s_gpgme_op_export_keys, 4);
+  rb_define_module_function (mGPGME, "gpgme_op_export_keys_start",
+			     rb_s_gpgme_op_export_keys_start, 4);
+#endif
   rb_define_module_function (mGPGME, "gpgme_op_import",
 			     rb_s_gpgme_op_import, 2);
   rb_define_module_function (mGPGME, "gpgme_op_import_start",
 			     rb_s_gpgme_op_import_start, 2);
+#ifdef HAVE_GPGME_OP_EXPORT_KEYS
+  rb_define_module_function (mGPGME, "gpgme_op_import_keys",
+			     rb_s_gpgme_op_import_keys, 2);
+  rb_define_module_function (mGPGME, "gpgme_op_import_keys_start",
+			     rb_s_gpgme_op_import_keys_start, 2);
+#endif
   rb_define_module_function (mGPGME, "gpgme_op_import_result",
 			     rb_s_gpgme_op_import_result, 1);
   rb_define_module_function (mGPGME, "gpgme_op_delete",
@@ -2253,6 +2420,11 @@ Init_gpgme_n (void)
 		   INT2FIX(GPGME_PROTOCOL_OpenPGP));
   rb_define_const (mGPGME, "GPGME_PROTOCOL_CMS",
 		   INT2FIX(GPGME_PROTOCOL_CMS));
+  /* This protocol was added in 1.2.0. */
+#ifdef GPGME_PROTOCOL_ASSUAN
+  rb_define_const (mGPGME, "GPGME_PROTOCOL_ASSUAN",
+		   INT2FIX(GPGME_PROTOCOL_ASSUAN))
+#endif
 
   /* gpgme_status_code_t */
   rb_define_const (mGPGME, "GPGME_STATUS_EOF",
@@ -2424,6 +2596,11 @@ Init_gpgme_n (void)
 #endif
   rb_define_const (mGPGME, "GPGME_KEYLIST_MODE_VALIDATE",
 		   INT2FIX(GPGME_KEYLIST_MODE_VALIDATE));
+  /* This flag was added in 1.2.0. */
+#ifdef GPGME_KEYLIST_MODE_EPHEMERAL
+  rb_define_const (mGPGME, "GPGME_KEYLIST_MODE_EPHEMERAL",
+		   INT2FIX(GPGME_KEYLIST_MODE_EPHEMERAL));
+#endif
 
   /* The available flags for status field of gpgme_import_status_t.  */
   rb_define_const (mGPGME, "GPGME_IMPORT_NEW", INT2FIX(GPGME_IMPORT_NEW));
@@ -2437,4 +2614,9 @@ Init_gpgme_n (void)
   /* The available flags for gpgme_op_encrypt.  */
   rb_define_const (mGPGME, "GPGME_ENCRYPT_ALWAYS_TRUST",
 		   INT2FIX(GPGME_ENCRYPT_ALWAYS_TRUST));
+  /* This flag was added in 1.2.0. */
+#ifdef GPGME_ENCRYPT_NO_ENCRYPT_TO
+  rb_define_const (mGPGME, "GPGME_ENCRYPT_NO_ENCRYPT_TO",
+		   INT2FIX(GPGME_ENCRYPT_NO_ENCRYPT_TO));
+#endif
 }
