@@ -11,6 +11,10 @@ Ruby-GPGME is a Ruby language binding of GPGME (GnuPG Made Easy).
 
 = Installation
 
+ $ gem install ruby-gpgme
+
+or
+
  $ ruby extconf.rb
  $ make
  $ make install
@@ -25,9 +29,10 @@ Ruby-GPGME is a Ruby language binding of GPGME (GnuPG Made Easy).
 
 = API
 
-Ruby-GPGME provides 3 levels of API.  The highest level API is close
-to the command line interface of GnuPG.  The lowest level API is close
-to the C interface of GPGME.
+Ruby-GPGME provides three levels of API.  The highest level API is
+close to the command line interface of GnuPG.  The mid level API looks
+object-oriented (or rubyish).  The lowest level API is close to the C
+interface of GPGME.
 
 == The highest level API
 
@@ -36,10 +41,20 @@ signature of the plaintext from stdin as follows.
 
  $ ruby -rgpgme -e 'GPGME.clearsign($stdin, $stdout)'
 
+== The mid level API
+
+The same example can be rewritten in the mid level API as follows.
+
+ $ ruby -rgpgme -e <<End  
+ ctx = GPGME::Ctx.new
+ plain = GPGME::Data.from_io($stdin)
+ sig = GPGME::Data.from_io($stdout)
+ ctx.sign(plain, sig, GPGME::SIG_MODE_CLEAR)
+ End
+
 == The lowest level API
 
-On the other hand, the same example can be rewritten in the lowest
-level API as follows.
+The same example can be rewritten in the lowest level API as follows.
 
  $ ruby -rgpgme -e <<End  
  ret = Array.new
@@ -53,23 +68,10 @@ level API as follows.
  End
 
 As you see, it's much harder to write a program in this API than the
-highest level API.  However, if you are already familier with the C
+higher level API.  However, if you are already familier with the C
 interface of GPGME and/or want to control detailed behavior of GPGME,
 it might be useful.
 
-== The mid level API
-
-There is another API which looks object-oriented.  It's easier to use
-than the lowest level API though, you should first consult the highest
-level API.
-
- $ ruby -rgpgme -e <<End  
- ctx = GPGME::Ctx.new
- plain = GPGME::Data.from_io($stdin)
- sig = GPGME::Data.from_io($stdout)
- ctx.sign(plain, sig, GPGME::SIG_MODE_CLEAR)
- End
-
 = License
 
 Copyright (C) 2003,2006,2007,2008,2009 Daiki Ueno
@@ -91,6 +93,7 @@ License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 =end
 module GPGME
+  VERSION = "1.0.8"
 end
 
 require 'gpgme_n'
@@ -123,6 +126,7 @@ def GPGME.decrypt(cipher, *args_options)
   args, options = split_args(args_options)
   plain = args[0]
 
+  check_version(options)
   GPGME::Ctx.new(options) do |ctx|
     cipher_data = input_data(cipher)
     plain_data = output_data(plain)
@@ -182,6 +186,7 @@ def GPGME.verify(sig, *args_options) # :yields: signature
   args, options = split_args(args_options)
   signed_text, plain = args
 
+  check_version(options)
   GPGME::Ctx.new(options) do |ctx|
     sig_data = input_data(sig)
     if signed_text
@@ -236,6 +241,7 @@ def GPGME.sign(plain, *args_options)
   args, options = split_args(args_options)
   sig = args[0]
 
+  check_version(options)
   GPGME::Ctx.new(options) do |ctx|
     ctx.add_signer(*resolve_keys(options[:signers], true, [:sign])) if options[:signers]
     mode = options[:mode] || GPGME::SIG_MODE_NORMAL
@@ -358,6 +364,7 @@ def GPGME.encrypt(recipients, plain, *args_options)
   cipher = args[0]
   recipient_keys = recipients ? resolve_keys(recipients, false, [:encrypt]) : nil
 
+  check_version(options)
   GPGME::Ctx.new(options) do |ctx|
     plain_data = input_data(plain)
     cipher_data = output_data(cipher)
@@ -412,6 +419,7 @@ def GPGME.list_keys(*args_options) # :yields: key
   raise ArgumentError, 'wrong number of arguments' if args_options.length > 3
   args, options = split_args(args_options)
   pattern, secret_only = args
+  check_version(options)
   GPGME::Ctx.new do |ctx|
     if block_given?  
       ctx.each_key(pattern, secret_only || false) do |key|
@@ -450,6 +458,7 @@ def GPGME.export(*args_options)
   args, options = split_args(args_options)
   pattern, key = args[0]
   key_data = output_data(key)
+  check_version(options)
   GPGME::Ctx.new(options) do |ctx|
     ctx.export_keys(pattern, key_data)
 
@@ -484,6 +493,7 @@ def GPGME.import(*args_options)
   args, options = split_args(args_options)
   key = args[0]
   key_data = input_data(key)
+  check_version(options)
   GPGME::Ctx.new(options) do |ctx|
     ctx.import_keys(key_data)
     ctx.import_result
@@ -506,6 +516,19 @@ module GPGME
   end
   module_function :split_args
 
+  def check_version(options = nil)
+    version = nil
+    if options.kind_of?(String)
+      version = options
+    elsif options.include?(:version)
+      version = options[:version]
+    end
+    unless GPGME::gpgme_check_version(version)
+      raise Error::InvalidVersion.new
+    end
+  end
+  module_function :check_version
+
   def resolve_keys(keys_or_names, secret_only, purposes = Array.new)
     keys = Array.new
     keys_or_names.each do |key_or_name|
@@ -656,6 +679,7 @@ module GPGME
     end
     class BadSignature < self; end
     class NoPublicKey < self; end
+    class InvalidVersion < self; end
   end
 
   def error_to_exception(err)   # :nodoc:
@@ -720,7 +744,6 @@ module GPGME
   private :error_to_exception
 
   class << self
-    alias check_version gpgme_check_version
     alias pubkey_algo_name gpgme_pubkey_algo_name
     alias hash_algo_name gpgme_hash_algo_name
   end
